import xml.etree.ElementTree as ET
import fnmatch

# Safe import of OrderedDict
try:
    from collections import OrderedDict
except ImportError:
    from ordered_dict_26 import OrderedDict

#################################

# Returns true if the value is an integer as written. Int examples: '5', '5e10'; Not int examples: '5.0', '5.5e10'
def is_int(value):

    try:
        string = str(value)
        no_dots = '.' not in string
        float_convert = float(string).is_integer()

        if no_dots and float_convert:
            return True

    except ValueError:
        pass

    return False


# Returns an integer if value is_int, otherwise returns a float
def cast_int_or_float(value):

    if is_int(value):
        return int(value)
    else:
        return float(value)


# Takes an XML ElementTree element and creates an OrderedDict from its children
def xml_to_dict(xml_element, skip_tags=[], tag_modify=(), skip_sub_tags=False):

    dict = OrderedDict()
    subiter = list(xml_element)

    for element in subiter:

        dict_value = None

        # Create elements with children as sub-dicts (e.g. Special_Constants or Field_Statistics)
        if list(element):

            if skip_sub_tags:
                continue
            else:
                dict_value = xml_to_dict(element, skip_tags, tag_modify, skip_sub_tags)

        # Create single elements
        elif element.text is not None:
            dict_value = element.text

        # Create an key/value pair from each tag/element value
        if dict_value:

            tag = element.tag
            if tag_modify:
                tag = tag.replace(tag_modify[0], tag_modify[1])

            skip = False
            for skip_tag in skip_tags:
                if fnmatch.fnmatch(tag, skip_tag):
                    skip = True
                    break

            if not skip:
                dict[tag] = dict_value

    return dict
