import xml.etree.ElementTree as ET
from xml.parsers.expat import ExpatError

from pds4_logging import logger_init

# Initialize the logger
logger = logger_init()

#################################


def read_label(filename, strip_namespaces=True, strip_extra_whitespace=True):

    """ Reads a PDS4 XML Label into an ElementTree object

        Parameters:

            strip_namespaces: bool
                Strips all namespaces from all tags
            strip_extra_whitespace: bool
                Strips whitespaces at the beginning and end of all element and attribute values

        Returns:

            label_xml_root: ElementTree
                Root element for read-in PDS4 label
    """

    # Read-in XML tree
    try:
        xml_tree = ET.iterparse(filename)
        logger.info('Processing label: ' + filename)
    except IOError:
        raise IOError('Unable to locate or read label file: ' + filename)

    # Adjust XML tree
    try:
        for event, elem in xml_tree:

            # Strip all namespaces if requested
            if strip_namespaces:

                if '}' in elem.tag:
                    elem.tag = elem.tag.split('}', 1)[1]

            # Strip whitespace in elements and attributes if requested
            if strip_extra_whitespace:

                subiter = ET.ElementTree(elem).getiterator()
                attribs = elem.attrib.iteritems()

                # Strip whitespaces in elements
                for elem_content in subiter:

                    if elem_content.text:
                        elem_content.text = elem_content.text.strip()

                # Strip whitespaces in attributes
                for name, value in attribs:
                    elem.attrib[name] = value.strip()

        label_xml_root = xml_tree.root

    except ExpatError:
        raise ExpatError('The requested label file does not appear contain valid XML: ' + filename)

    return label_xml_root

# import re, fnmatch
# regex = fnmatch.translate('*.txt')
# pattern = re.compile(regex_fnmatch)
# matches = pattern.search(string)
