import struct

from pds4_logging import logger_init
import pds4_read_tables_ascii

# Safe import of numpy (not required)
try:
    import numpy as np
except ImportError:
    pass

# Initialize the logger
logger = logger_init()

#################################

# Use in reading routine, not here
# Append FieldBit if another FieldBit has the same name
# if field_bit.name in field_bits:
#
#     if isinstance(field_bits[field_bit.name], FieldBit):
#         field_bits[field_bit.name] = tuple(field_bits[field_bit.name])
#
#     field_bits[field_bit.name] += (field_bit, )
#
# # Append new FieldBit
# else:
#     field_bits[field_bit.name] = field_bit

def data_type_convert(data_type, byte_data, use_numpy=False):

    struct_type = {'IEEE754MSBSingle': ('>f', 'float32'),
                   'IEEE754MSBDouble': ('>d', 'float64'),
                   'SignedMSB2': ('>h', 'int16'),
                   'SignedMSB4': ('>l', 'int32'),
                   'SignedMSB8': ('>q', 'int64'),
                   'UnsignedMSB2': ('>H', 'uint16'),
                   'UnsignedMSB4': ('>L', 'uint32'),
                   'UnsignedMSB8': ('>Q', 'uint64'),
                   'ComplexMSB8': ('>ff', 'complex64'),
                   'ComplexMSB16': ('>dd', 'complex128'),

                   'IEEE754LSBSingle': ('<f', 'float32'),
                   'IEEE754LSBDouble': ('<D', 'float64'),
                   'SignedLSB2': ('<h', 'int16'),
                   'SignedLSB4': ('<l', 'int32'),
                   'SignedLSB8': ('<q', 'int64'),
                   'UnsignedLSB2': ('<H', 'uint16'),
                   'UnsignedLSB4': ('<L', 'uint32'),
                   'UnsignedLSB8': ('<Q', 'uint64'),
                   'ComplexLSB8': ('<ff', 'complex64'),
                   'ComplexLSB16': ('<dd', 'complex128'),

                   'SignedByte': ('b', 'int8'),
                   'UnsignedByte': ('B', 'uint8')
                   # 'SignedBitString': ('', ''),   # Currently unhandled
                   # 'UnsignedBitString': ('', '')

                   }.get(data_type, None)

    # Convert binary data
    if struct_type is not None:

        # Convert complex binary data types
        if 'Complex' in data_type:

            for i, datum in enumerate(byte_data):
                unpacked_data = struct.unpack(struct_type[0], datum)
                byte_data[i] = complex(unpacked_data[0], unpacked_data[1])

        # Convert all other binary data types
        else:

            for i, datum in enumerate(byte_data):
                byte_data[i] = struct.unpack(struct_type[0], datum)[0]

        # Convert data to numpy data types if requested
        if use_numpy:

            np_data_type = getattr(np, struct_type[1])
            byte_data = np.asarray(byte_data, dtype=np_data_type)

    # Convert ASCII data
    else:
        byte_data = pds4_read_tables_ascii.data_type_convert(data_type, byte_data, use_numpy)

    return byte_data

